# DestinE utils

Collection of scripts to deploy, run and analyze the performance of the IFS-NEMO

- [DestinE utils](#destine-utils)
  - [Requeriments](#requeriments)
  - [Versions](#versions)
  - [Installation / Compilation IFS-Bundle](#installation--compilation-ifs-bundle)
    - [MN5 Complete deployment](#mn5-complete-deployment)
    - [MN5 build from sources](#mn5-build-from-sources)
    - [MN5 Manual installation](#mn5-manual-installation)
  - [NEMO-Standalone Installation/Compilation](#nemo-standalone-installationcompilation)
  - [Running](#running)
    - [IFS-Bundle](#ifs-bundle)
    - [NEMO-Standalone](#nemo-standalone)
  - [Performance](#performance)
    - [IFS-Bundle](#ifs-bundle-1)
    - [NEMO-Standalone](#nemo-standalone-1)


## Requeriments

* ECMWF Git account and SSH connection on your workstaion.
* Have MN5 transfer nodes in the ssh configuration of your workstaion.
* Clone this repo in your local machine & MN5

## Versions
Scrip to generate a file with the version (repo, branch and commit hash)
of each component of the IFS-BUNDLE.
```bash
Usage: ./check_ifs_bundle_version.sh <ifs-bundle-dir>
```
## Installation / Compilation IFS-Bundle
Currently, only MN5 is supported by this scripts. The compilation is based
on CMake, but with an ECMWF wrapper called ECBuild and a Python script to 
manage all the dependencies (called ifs-bundle). All the compilations of this
scripts are described in the following subsections:

### MN5 Complete deployment

Follow the instructions below to install the model "by hand". Remember that 
we need to copy the sources to MN5 from your workstation and then 
submit the compilation. Check the environment file for MN5 in order to
select the working RAPS, IFS-SOURCE and IFS-BUNDLE:

```bash
# RAPS: Responsible vcorreal
export IFS_BUNDLE_RAPS_GIT="ssh://git@git.ecmwf.int/~esp0512/raps-optimizations.git"
export IFS_BUNDLE_RAPS_VERSION="features/mn5"
# IFS-SOURCE: Responsible cpena
export IFS_BUNDLE_IFS_SOURCE_GIT="ssh://git@git.ecmwf.int/~pacp/ifs-source-de.git"
export IFS_BUNDLE_IFS_SOURCE_VERSION="MN5"
# This skips the downloading of the ORCA grids during the compilation (need at MN5)
export IFS_BUNDLE_ATLAS_ORCA_CMAKE="ENABLE_RETRIEVE_ORCA_DATA=OFF ENABLE_INSTALL_ORCA_DATA=OFF"

```

> It is pending to get all this changes pushed into a stable
> IFS-Bundle tag

1. Using your local machine, go to `installation/ifs-bundle` path.
2. Execute `clone_sources.sh`. You need to select the ifs-bundle repository 
with `--repo`.

> You can select the desired branch of the repo with --branch and a 
> bundle environment with --bundle-env.

3. By default, the name of the installation is the repo name followed by the 
branch name and the commit hash. You can select another name with `--name`.

4. Select the MN5 path to move the sources with `--mn5-dir`, using the
syntax: `mn5-transfer1:/your/desired/path`

5. A prompt will resume all the choices before submit the start of the process.

6. Open a terminal to MN5 login node and go to `installation/ifs-bundle`.

7. Once you have the sources in MN5, you can submit the compilation in two ways
using the same `compile-mn5-gpp.slurm` script. This script requires two 
positional arguments, first the root of your ifs-bundle (`--mn5-dir` parameter)
and second the arch (is optional, by default it uses 
the `arch/eurohpc/mn5-gpp/default`):

    ```bash
    # Interactive compilation
    bash compile-mn5-gpp.slurm /gpfs/projects/bsc32/DestinE/models/ifs-nemo/perfomance [arch file]
    # SLURM compilation 
    sbatch --qos <your_queue> compile-mn5-gpp.slurm /gpfs/projects/bsc32/DestinE/models/ifs-nemo/perfomance [arch file]
    ```

The following code fence show a complete example:
````bash
# LOCAL MACHINE (WITH ECMWF GIT SSH CONFIGURED), at destine-utils/installation

./clone_sources.sh --repo ssh://git@git.ecmwf.int/~esp0512/ifs-bundle-performance.git \
  --branch feature/mn5 \
  --mn5-dir mn5-transfer1:/gpfs/projects/bsc32/DestinE/models/ifs-nemo/perfomance \
  --bundle-env mn5-gpp-env 

# AT MN5 login node (any login node), at destine-utils/installation
sbatch --qos <your_queue> \
  compile-mn5-gpp.slurm /gpfs/projects/bsc32/DestinE/models/ifs-nemo/perfomance

````
### MN5 build from sources
If you need to build the IFS-Bundle from the sources, you need to use the 
`submit-mn5-gpp.sh` script, follow these steps:

0. If you need to upload the sources, you need to clone the ifs-bundle using
the `clone_sources.sh` script, generating a tar and sending it to MN5:

````bash
./clone_sources.sh --repo ssh://git@git.ecmwf.int/~esp0512/ifs-bundle-performance.git --branch feature/mn5 --bundle-env mn5-gpp-env --name stable-ifs-bundle-mn5
cd stable-ifs-bundle-mn5
tar -cf stable-ifs-bundle-mn5.tar .
scp stable-ifs-bundle-mn5.tar mn5-transfer1:/your/path/for/sources
````

1. Once you have located the sources, go to the `destine-utils/installation/ifs-bundle` folder.
2. The `submit-mn5-gpp.sh` sets the install directory, checks for the arch file and then
uncompress the sources and executes the `compile-mn5-gpp.slurm` script to compile the model:

```bash
Usage: ./submit-mn5-gpp.sh <source (tar or tar.gz)> <install_dir> <running_method>
OPTIONAL: <arch_setup> (otherwise it will use the mn5-gpp default)
arch setup: path like arch/eurohpc/mn5-gpp/default
running method: interactive | mn5_queue (see bsc_queues command)
```
> The source path must be an absolute path!

### MN5 Manual installation
The following steps are required to install the IFS-Bundle model:

1. Upload the sources into MN5 (using the transfer machine)
2. Re-run `./ifs-bundle create` to refresh symlinks. Remember to select the env
to avoid the grid download.
3. Configure the build type using: 
  ````bash
  ./ifs-bundle build --arch ${ARCH_SETUP} --with-single-precision \
            --with-double-precision-nemo --nemo-version=V40 \
            --nemo-grid-config=eORCA1_GO8_Z75 --nemo-ice-config=SI3 \
            --with-multio-for-nemo-sglexe --dry-run --verbose \
            --nemovar-grid-config=ORCA1_Z42 --nemovar-ver=DEV
  ````
4. Go into the `build` directory and start the compilation:
  ````bash
  cd build && ./configure
  source env.sh && make -j 33
  ````
5. Initialize RAPS, going into the `source/raps` directory and running `source initbm`
6. Add the atlas grids from the common dir
: `cd $INSTALL_DIR/build/share/atlas/grids/orca && ln -svf $ATLAS_GRIDS`

## NEMO-Standalone Installation/Compilation
To deploy de NEMO standalone model, you can use the

## Running

### IFS-Bundle
To run the model, the RAPS (located into the source/raps) of the installation dir
is required (this script prepare the input files, I/O setup, etc...). To change
the running method or add wrappers to the execution, you need tune the
`source/raps/bin/hres` bash script.

This utils provide a bash interface to submit executions changing the setup 
(nodes, I/O setup, threads per task, etc). This script is under the `running` 
folder.

### NEMO-Standalone


## Performance
### IFS-Bundle

This folder contains several python/bash scripts for gather performance 
information of the runs:
* `gather_results.sh` : Gets the current status for all the executions in
a running directory, SDPD reported are erronoeus.
* `get_fc_days.py`: Gets the SDPD for all the executions of a running directory.

### NEMO-Standalone
> WIP