
import os, re


def find_rundir_folders(root_path):
    rundir_folders = []

    for dirpath, _, filenames in os.walk(root_path):
        if "ifs.err" in filenames:
            rundir_folders.append(dirpath)

    return rundir_folders

def get_sdpd_from_rundir(rundir, from_ts=-1, to_ts=-1):
    try:
        # locate simualted_seconds_per_step
        time_per_step = get_simualted_seconds_per_step(rundir)

        if time_per_step is None:
            return None
        else:
            pass
            #print(">Uses", time_per_step, "as simulated time per step")

        # parse ifs.stat file:
        data = {}
        inst_sdpd = []
        acum_time = float(0.0)
        instantaneous_sdpd = float(0.0)
        steps_completed=0
        with open(os.path.join(rundir, 'ifs.stat')) as ifs_steps:
            for line in ifs_steps.readlines():
                # parsed[3] -> step, parsed[2] -> STEPO - DYNFPOS
                parsed = list(filter(lambda i : i != "", line.split(" ")))
                if from_ts == -1 or int(parsed[3]) >= from_ts:
                    time_step_id = parsed[3]+"_"+parsed[2]
                    step_time = float(parsed[6])
                    steps_completed = int(parsed[3])
                    acum_time += step_time

                    if steps_completed > 0:
                        instantaneous_sdpd = ( steps_completed * time_per_step ) / acum_time

                    if to_ts != -1 and steps_completed > to_ts:
                        break

                    data[time_step_id] = step_time
                    inst_sdpd.append(instantaneous_sdpd)
            # Compute SDPD:
            sdpd = (steps_completed*time_per_step)/acum_time
        return float(sdpd)

    except Exception as e:
#        print("Error processing", rundir)
        return -1.0

def get_simualted_seconds_per_step(rundir):
    # grep time per step from fort.4:
    pattern = re.compile(r'TSTEP=(\d+)')
    with open(os.path.join(rundir, 'fort.4')) as ifs_namelist:
        for line in ifs_namelist.readlines():
            match = pattern.search(line)
            if match:
                time_per_step = int(match.group(1))
                return time_per_step
    
def get_hardware_setup(rundir):
    regex_num_nodes = re.compile(r'SLURM_NNODES=(\d+)')
    regex_mpi_per_node = re.compile(r'SLURM_TASKS_PER_NODE=(\d+)')
    regex_omp_per_node = re.compile(r'SLURM_CPUS_PER_TASK=(\d+)')
    nodes, mpi, omp = 0,0,0
    try:
        with open(os.path.join(rundir, 'ifs.env')) as ifs_env:
            for line in ifs_env.readlines():
                match_nodes = regex_num_nodes.search(line)
                match_mpi_per_node = regex_mpi_per_node.search(line)
                match_omp_per_node = regex_omp_per_node.search(line)
                if match_nodes:
                    nodes = int(match_nodes.group(1))
                if match_mpi_per_node:
                    mpi = int(match_mpi_per_node.group(1))
                if match_omp_per_node:
                    omp = int(match_omp_per_node.group(1))
    except Exception as e:
        print(e)
    
    return nodes, mpi, omp
    
